/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   Use this program to experiment with two-dimensional transformations.
<p>
   Try mixing the ordering of rotations, translations, and scales.
*/
public class TransformationTester
{
   public static void main(String[] args)
   {
      // Create several 2D Model objects.
      Model m1 = new GRSModel(new File("assets/grs/bronto.grs"));
      Model m2 = new GRSModel(new File("assets/grs/usa.grs"));
      Model m3 = new GRSModel(new File("assets/grs/rex.grs"));
      Model m4 = new GRSModel(new File("assets/grs/vinci.grs"));

      // Pick a model from the above models.
      Model m = m1;

      // Set the model's color.
      m.setColor(Color.black);

      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the model to the Scene.
      Position position = new Position(m);
      scene.positionList.add(position);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-1, +1, -1, +1, 24, 24);
      axes.setColor( Color.red );
      scene.positionList.add(new Position(axes));


      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);


      // Use orthographic projection
      scene.camera.perspective = false;

      for (int i = 0; i <= 36; i++)
      {
         position.matrix2Identity();
         // try changing the order of these transformations
         position.scale(0.33); // scale the model to fit in the view rectangle
         position.rotateZ(10*i);
         position.translate(1, 0, 0);

         // Render again.
         fb.clearFB(Color.lightGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_TransformationTester_Frame%02d.ppm", i));
      }
   }
}