/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its center, with its center at the origin.
<p>
   Demonstrate the transformations of the coordinate system.
<p>
   In the last example file, we said that this code
   <pre>{@code
      square.matrix2Identity();
      square.rotateZ(6*i);
      square.translate(-0.5, -0.5, 0);
   }</pre>
   when read from the top down, should be read as a sequence of
   transformations of the coordinate system. (When it is read from
   the bottom up, it should be read as a sequence of transformations
   of vertices.) This program draws a visualization of the coordinate
   system being transformed.
<p>
   In this animation, each frame of the last example animation is turned
   into four frames  The first frame represents this line of code
   <pre>{@code
      square.matrix2Identity();
   }</pre>
   which restores the coordinates system to the original one. The second
   frame represents this line of code
   <pre>{@code
      square.rotateZ(6*i);
   }</pre>
   which rotates the coordinate system. The third frame represents this
   line of code
   <pre>{@code
      square.translate(-0.5, -0.5, 0);
   }</pre>
   which translates the rotated coordinate system. The last frame then
   draws the square model in the last coordinate system.
*/
public class Example_04b
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      axes.setColor( Color.red );
      // Create a Position for the axes.
      Position axes_p = new Position(axes);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);
      // Push the axes away from where the camera is.
      axes_p.translate(0, 0, -2);

      // Create two shorter sets of x and y axes.
      Model sAxes1 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes2 = new Axes2D(-0.5, +0.5, -0.5, +0.5, 2, 2);
      sAxes1.setColor( new Color(150, 0, 0) );
      sAxes2.setColor( new Color(100, 0, 0) );
      Position sAxes1_p = new Position(sAxes1);
      Position sAxes2_p = new Position(sAxes2);
      sAxes1_p.translate(0, 0, -2);
      sAxes2_p.translate(0, 0, -2);
      // Add the shorter axes to the Scene.
      scene.addPosition(sAxes1_p, sAxes2_p);

      // Create a Model of a square.
      Model square = new Model();
      square.addVertex(new Vertex(0, 0, 0));
      square.addVertex(new Vertex(1, 0, 0));
      square.addVertex(new Vertex(1, 1, 0));
      square.addVertex(new Vertex(0, 1, 0));
      square.addLineSegment(0, 1);
      square.addLineSegment(1, 2);
      square.addLineSegment(2, 3);
      square.addLineSegment(3, 0);
      // Color the square blue.
      square.setColor( Color.blue );
      // Create a Position for the square.
      Position square_p = new Position(square);
      // Add the square to the Scene.
      scene.addPosition(square_p);

      // Rotate the square around its center, with its center at the origin.
      for (int i = 0; i < 60; i++)
      {
         sAxes1_p.matrix2Identity();
         sAxes1_p.translate(0, 0, -2);
         sAxes2_p.matrix2Identity();
         sAxes2_p.translate(0, 0, -2);

         square_p.matrix2Identity();
         // Push the square away from where the camera is.
         square_p.translate(0, 0, -2);

         sAxes1_p.rotate(6*i, 0,0,1);

         sAxes2_p.rotate(6*i, 0,0,1);
         sAxes2_p.translate(-0.5, -0.5, 0);

         square_p.rotate(6*i, 0,0,1);
         square_p.translate(-0.5, -0.5, 0);

         sAxes1.visible = false;
         sAxes2.visible = false;
         square.visible = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03da.ppm", i));

         sAxes1.visible = true;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03db.ppm", i));

         sAxes2.visible = true;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03dc.ppm", i));

         square.visible = true;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04b_Frame%03dd.ppm", i));
      }
   }
}
