/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around its center, with its center at the origin.
<p>
   Do this by "accumulating" rotations.
<p>
   The last animation showed how the coordinate system gets transformed
   by the code. Notice in that animation that we always "go back" to the
   original coordinate system and then rotate it. This example shows
   we don't need to go back to the original coordinate system. We can "save"
   the rotated coordinate system, translate it, then draw, then "return" to
   the rotated coordinate system, save it again, translate it again, then
   draw, etc.
<p>
   This is not a better way to create the animation since it is harder to
   read. But sometimes this idea of "saving" and "restoring" a coordinates
   system can be useful.
*/
public class Example_04c
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      axes.setColor( Color.red );
      // Create a Position for the axes.
      Position axes_p = new Position(axes);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);
      // Push the axes away from where the camera is.
      axes_p.translate(0, 0, -2);

      // Create a Model of a square.
      Model square = new Model();
      square.addVertex(new Vertex(0, 0, 0));
      square.addVertex(new Vertex(1, 0, 0));
      square.addVertex(new Vertex(1, 1, 0));
      square.addVertex(new Vertex(0, 1, 0));
      square.addLineSegment(0, 1);
      square.addLineSegment(1, 2);
      square.addLineSegment(2, 3);
      square.addLineSegment(3, 0);
      // Color the square blue.
      square.setColor( Color.blue );
      // Create a Position for the square.
      Position square_p = new Position(square);
      // Add the square to the Scene.
      scene.addPosition(square_p);

      // Push the square away from where the camera is.
      square_p.translate(0, 0, -2);

      // Rotate the square around its center, with its center at the origin.
      Matrix save = square_p.modelMatrix;   // save the coordinate system
      for (int i = 0; i < 60; i++)
      {
         square_p.translate(-0.5, -0.5, 0); // update the coordinate system

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_04c_Frame%03d.ppm", i));

         square_p.modelMatrix = save; // restore the coordinate system
         square_p.rotate(6, 0,0,1);   // accumulate a rotation
         save = square_p.modelMatrix; // save the coordinate system again
      }
   }
}
