/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter {@link P} around the tip of its front.
<p>
   See the animation <a href="../Example_12.gif">Example_12.gif</a>.
<p>
   <a href="../Example_12.gif"><img src="../Example_12.gif" alt="Example_12.gif"></a>
*/
public class Example_12
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-2, +2, -2, +2, 8, 8);
      // Color them red.
      axes.setColor( Color.red );
      // Push the axes away from where the camera is.
      Position axes_p = new Position(axes);
      axes_p.translate(0, 0, -2);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      modelP.setColor( new Color(50, 150, 170) );
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Rotate the letter P around the tip of its front.
      for (int i = 0; i < 60; i++)
      {
         modelP_p.matrix2Identity();
         // Push the model away from where the camera is.
         modelP_p.translate(0, 0, -2);

         // Transform the model.
         modelP_p.translate( 1.0,  0.7, 0);
         modelP_p.rotate(i*6, 0,0,1);
         modelP_p.translate(-1.0, -0.7, 0);

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_12_Frame%03d.ppm", i));
      }
   }
}
