/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter {@link P} down the x-axis.
<p>
   See the animation <a href="../Example_26_puzzle.gif">Example_26_puzzle.gif</a>.
<p>
   <a href="../Example_26_puzzle.gif"><img src="../Example_26_puzzle.gif" alt="Example_26.gif"></a>
*/
public class Example_26
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-5, +5, -5, +5, 20, 20);
      // Color them red.
      axes.setColor( Color.red );
      // Push the axes away from where the camera is.
      Position axes_p = new Position(axes);
      axes_p.translate(0, 0, -5);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      modelP.setColor( Color.blue );
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);

      // Rotate the letter P down the x-axis.

      // do the first 90 degrees in steps of 3 degrees
      for (int i = 0; i <= 30; i++)
      {
         modelP_p.matrix2Identity();  // Undo all the previous transformations.
         // Push the model P away from where the camera is.
         modelP_p.translate(0, 0, -5);



         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_26_Frame%03d.ppm", i));
      }

      // do the next 180 degrees in steps of 3 degrees
      for (int i = 1; i <= 60; i++)
      {
         modelP_p.matrix2Identity();  // Undo all the previous transformations.
         // Push the model P away from where the camera is.
         modelP_p.translate(0, 0, -5);



         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_26_Frame%03d.ppm", 30+i));
      }

      // do the next 180 degrees in steps of 3 degrees
      for (int i = 1; i <= 60; i++)
      {
         modelP_p.matrix2Identity();  // Undo all the previous transformations.
         // Push the model P away from where the camera is.
         modelP_p.translate(0, 0, -5);



         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_26_Frame%03d.ppm", 90+i));
      }

      // do the next 180 degrees in steps of 3 degrees
      for (int i = 1; i <= 60; i++)
      {
         modelP_p.matrix2Identity();  // Undo all the previous transformations.
         // Push the model P away from where the camera is.
         modelP_p.translate(0, 0, -5);



         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_26_Frame%03d.ppm", 150+i));
      }

      // do the last 90 degrees in steps of 3 degrees
      for (int i = 1; i <= 30; i++)
      {

         modelP_p.matrix2Identity();  // Undo all the previous transformations.
         // Push the model P away from where the camera is.
         modelP_p.translate(0, 0, -5);



         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_26_Frame%03d.ppm", 210+i));
      }
   }
}
