/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
   Scale the letter {@link P} larger as its left edge
   forms a 45 degree diagonal from the positive
   y-axis to the positive x-axis.
<p>
   See the animation <a href="../Example_35.gif">Example_35.gif</a>.
<p>
   <a href="../Example_35.gif"><img src="../Example_35.gif" alt="Example_35.gif"></a>
*/
public class Example_35
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Create a set of x and y axes.
      Model axes = new Axes2D(-4, +4, -4, +4, 16, 16);
      // Color them red.
      axes.setColor( Color.red );
      // Push the axes away from where the camera is.
      Position axes_p = new Position(axes);
      axes_p.translate(0, 0, -4);
      // Add the axes to the Scene.
      scene.addPosition(axes_p);

      // Create a Model of the letter P.
      P modelP = new P();
      // Color the P blue.
      modelP.setColor( Color.blue );
      // Add the model to the Scene.
      Position modelP_p = new Position(modelP);
      scene.addPosition(modelP_p);


      // Scale the letter P larger as its left edge
      // forms a 45 degree diagonal from the positive
      // y-axis to the positive x-axis.
      for (int i = 2; i <= 64; i++)
      {
         modelP_p.matrix2Identity();
         // Push the model away from where the camera is.
         modelP_p.translate(0, 0, -4);

         double scale = Math.pow(i/2.0, 0.5);
         modelP_p.scale(scale, scale, 1);
         modelP_p.translate(1/Math.pow(2, 0.5), 0, 0);
         modelP_p.rotate(180, 0,1,0);
         modelP_p.rotate(-45, 0,0,1);
/*
         modelP_p.rotate( 45, 0,0,1);
         modelP_p.rotate(180, 0,1,0);
         double scale = Math.pow(i/2.0, 0.5);
         modelP_p.scale(scale, scale, 1);
         modelP_p.translate(-0.5, -0.5, 0);
*/
         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_35_Frame%03d.ppm", i));
      }
   }
}
